#!/usr/bin/env python2
# encoding: utf-8

"""
    Partial python implementation of the published docconverter.pro Web API

    requires requests module https://pypi.python.org/pypi/requests
        - to install, "pip install requests" in (windows) command line or (MacOS) Terminal Window

    Tested:
        MacOS 10.12.5 (Sierra): Python 2.7.8, 2.7.10
        MacOS 10.12.6 (High Sierra): Python 2.7.10, 2.7.14
        MacOS 10.12.5 (Sierra): Python 3.5.1
        MacOS 10.12.6 (High Sierra): Python 3.6.4

        Windows 10: Python 2.7.11
        Windows 10: Python 3.5.1
		Windows 10: Python 3.6
"""

try:
    import requests
    import requests.packages.urllib3

    from requests.exceptions import ( RequestException,
                                      ConnectionError,
                                      HTTPError,
                                      URLRequired,
                                      TooManyRedirects,
                                      ConnectTimeout,
                                      ReadTimeout,
                                      Timeout
                                    )
except ImportError:
    raise

# Turn off requests warnings about being an insecure site - better would be to fix it :-(
requests.packages.urllib3.disable_warnings()

# the URL we'll connect to
DP_API_BASE_URL = 'https://api.docconverter.pro/'

class api_docconverter_pro:
    """
    A class which implements a docconverter.pro (dp) SINGLE FILE web api connection
    
    Usage:

        1. Instantiate a connection, e.g.
            my_dp_connection = api_docconverter_pro(user_id=<my_user_id>, password=<my_password>)
                where the user_id and password are those for your docconverter.pro account
            "set_verbose" defaults to 'False'; set to 'True' for verbose execution tracing

        2. send a SINGLE file to be converted and receive the results
            response = dp.convert_and_return_as_html(ms_doc_file_path, my_template_id, verbose=True)
                where:  ms_doc_file_path is the path to the MS Word file to convert,
                        my_template_id is the conversion template to use from the list in your docconverter.pro account
                        response is the result returned from docconverter.pro (see the method definition for more info)
                        verbose turns trace printing on/off
            "set_verbose" defaults to value set during instantiation; 
                set to 'True' for individual verbose conversion tracing

            Note: "convert_and_return_as_zip" conversion is not implemented

        A simple test is provided at the bottom of this file
    """

    # Class Variable default initialization
    dp_token = None
    verbose = False

    # --------------------------------------------------------------
    #
    def __init__(self, user_id='your docconverter.pro user id', password='your docconverter.pro password', set_verbose=None):

        # Make sure set_verbose is a boolean or None as expected
        assert(isinstance(set_verbose, bool) or None), 'api_docconverter_pro: init: set_verbose must be of type Boolean'

        if set_verbose:
            self.verbose = set_verbose

        self.dp_token = None
        try:
            self.dp_token = self.__get_dp_token(user_id, password)
        except RequestException as why:
            if self.verbose:
                print ('RequestException: %s' % why)
            raise
        except Exception as why:
            if self.verbose:
                print ('unknown error: %s' % why)
                raise

    # --------------------------------------------------------------
    #
    def wp_set_verbose(verbose_setting):
        self.verbose = verbose_setting

    # --------------------------------------------------------------
    #
    def __get_dp_token(self, user_id, password):
        """
        This routine returns an authorization token from docconverter.pro
		Token is valid for 14 days, do not get it per each request. Store it in any persistent storage or cache and reuse it.
        If the token request fails, a Value Error is raised
        """

        MAX_RETRY = 3

        dp_token_url = DP_API_BASE_URL + "/Token"
        request_data = 'grant_type=password&username=' + user_id + '&password=' + password
        request_headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        dp_token = None

        for i in range(MAX_RETRY):
            try:
                response = requests.post(dp_token_url, 
                                         data=request_data, 
                                         headers=request_headers, 
                                         timeout=(3.05, 27))

                dp_token = response.json()

                if 'access_token' in dp_token:
                    return dp_token['access_token']
                elif 'error' in dp_token:
                    raise ValueError('get_dp_token failed. ' \
                                        + '%s - %s' % (dp_token[u'error'], dp_token['error_description']))
                else:
                    raise ValueError('api_docconverter_pro: get_dp_token failed. ' + 'Unknown error = response was %s', response)
            except ConnectTimeout:
                #ignore. We're retrying
                continue
            except RequestException as why:
                if self.verbose:
                    print ('api_docconverter_pro: get_dp_token failed. msg= %s' % why)
                raise

        if self.verbose:
            print ('api_docconverter_pro: get_dp_token failed. Max retries exceeded - Connection Timeout')

        raise ConnectTimeout('api_docconverter_pro: get_dp_token failed. - Connection Timeout')

    # --------------------------------------------------------------
    #
    def convert_and_return_as_html(self, ms_doc_file_path, template='Convert to HTML5', set_verbose=None):
        """
        ... from docconverter.pro docs ---
        This routine converts a SINGLE doc file and returns the result as a dictionary
        which includes:
            'status'        : the returned status code
            'data'          : the raw bytes returned for the conversion
            'content-type'  : the returned type of the data
            'content-length': the returned length, in bytes, of the data
        """

        # Make sure set_verbose is a boolean or None as expected
        assert(isinstance(set_verbose, bool) or None), 'api_docconverter_pro: set_verbose must be of type Boolean'
        assert(self.dp_token is not None), 'api_docconverter_pro: no docconverter.pro authorization token'

        verbose = set_verbose or self.verbose

        dp_convert_url   = DP_API_BASE_URL + '/api/converter/convertdoc'
        dp_authorization = {"Authorization": "Bearer " + self.dp_token}
        dp_params        = {'template':template, 'returnHtml': 'true', 'returnData': 'true'}

        try:
            file_to_convert = {'file': open(ms_doc_file_path, 'rb')}
            if verbose:
                print ('   ... MS doc(x)file_path = %s' % (ms_doc_file_path))

            # Send the request to docconverter.pro
            response = requests.post(dp_convert_url, 
                                     headers=dp_authorization, 
                                     data=dp_params, 
                                     files=file_to_convert, 
                                     timeout=(3.05, 27))
            #
            # We're back with a response from the request
            #
            if response.status_code == requests.codes.ok:
                # All went well, return the result
                return {'status': response.status_code, 
                        'data': response.content, 
                        'content-type': response.headers['content-type'], 
                        'content-length': response.headers['content-length']
                       }
            else:
                if verbose:
                    print ('Convert Result Fail:', response.status_code)

                # Problem, so just return the status code and "None" for everything else
                return {'status': response.status_code, 
                        'data': None, 
                        'content-type': None, 
                        'content-length': None
                       }
        except IOError as why:
            if verbose:
                print ('IOError - %s' % why)
            raise
        except requests.exceptions as why:
            if verbose:
                print ('Conversion Failed - %s' % why)
            raise

# End of api_docconverter_pro class def

#
# --------------------------------------------------------------
#
if __name__ == '__main__':
    # Quick test...

    # specify 
    #   1. the directory path of a test word file to convert and the html file to create
    #      e.g. any doc(x) file in your working directory

    ms_doc_file_path         = './TestFile.docx'
    converted_html_file_path = './TestFile.html'

    #   2. your docconverter.pro account info
    my_user_id     = '??'
    my_password    = '??'
	
    #   3. the conversion template to use (from list on docconverter.pro site)
    #      e.g. 'Convert to HTML5'
    my_template_id = 'Convert to HTML5'

    try:
        # instantiate an api_docconverter_pro instance (dp connection via docconverter.pro web api)
        #  - gets the authorization token
        dp = api_docconverter_pro(user_id=my_user_id, password=my_password, set_verbose=True)

        # we have a connection, so...
        try:
            # send the the test file to be converted
            print ('Cleaning', ms_doc_file_path)
            response = dp.convert_and_return_as_html(ms_doc_file_path, my_template_id, set_verbose=True)

            # We're back after the conversion. 
            # Was it ok?
            if response['status'] == requests.codes.ok:
                # Create a new file and write the html conversion (response['data']) into it
                # Note: Being inside the "with" means the file will be closed automatically
                with open(converted_html_file_path, mode='wb') as converted_file:
                    converted_file.write(response['data'])
                print ('>>> Success! Status = %s' % response['status'])
            else:
                print ('>>> Conversion Failed: Status Code:', response['status'])
        except IOError as why:
            print ('IOError:', why)

        except RequestException as why:
            print ('Requests Error:', why)

    except ConnectionError as why:
        print ('Connection Failed:', why)

    except ValueError as why:
        print ('Value Error:', why)

